/***************************************************************************************
  *
  * IOCTLs.h - Definition File for Kernel Driver IOCTLs
  *   Note: This is the ONLY Header File that can be included both by Kernel and Win32 modules.
  *
  * Copyright (c)2003 Vito Plantamura. All Rights Reserved.
  *
***************************************************************************************/

#pragma once

//========================
// Required Header Files.
//========================

#ifndef VPCKNT_SYSMODULE
	#include <winioctl.h>
	#include "NdisHook.h"
#endif

//=====================
// IOCTLs Definitions.
//=====================

#define IOCTL_VPCKNT_INITIALIZE_HOOK		CTL_CODE(IOCTL_UNKNOWN_BASE, 0x0800, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_VPCKNT_GET_VERSION			CTL_CODE(IOCTL_UNKNOWN_BASE, 0x0801, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_VPCKNT_SUBMIT_OALIST			CTL_CODE(IOCTL_UNKNOWN_BASE, 0x0802, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_VPCKNT_GET_NEXT_PACKET		CTL_CODE(IOCTL_UNKNOWN_BASE, 0x0803, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)

//=================================
// IOCTLs Related Data Structures.
//=================================

#pragma pack(push, 1)

	// --> IOCTL_VPCKNT_INITIALIZE_HOOK <--

	typedef struct _INITIALIZE_HOOK_INPUT
	{
		// ... user notification event handle ...

		HANDLE		hNotificationEvent;

		// ... packets queue length ...

		DWORD		dwPacketsBuffMaxItems;

		// ... loopback trick ...

		DWORD		dwLtLoopbackAdapterIpAddress; // tipically: 0xA9FE1981; zero if lt is disabled.
		DWORD		dwLtTrickNatIpAddress; // tipically: 0xA9FE1980; zero if lt is disabled.

	} INITIALIZE_HOOK_INPUT, *PINITIALIZE_HOOK_INPUT;

	typedef struct _INITIALIZE_HOOK_OUTPUT
	{
		// ...status...

		ULONG		bNdisHookSucceeded;

		// ...info...

		DWORD		dwProtocolListBufferSize;

		// ...data...

			//
			// PROTOCOLLIST_BUFFER follows.
			//

	} INITIALIZE_HOOK_OUTPUT, *PINITIALIZE_HOOK_OUTPUT;

	#define INITIALIZE_HOOK_INPUT_SIZE				( sizeof( INITIALIZE_HOOK_INPUT ) )
	#define INITIALIZE_HOOK_OUTPUT_MAXSIZE			( sizeof( INITIALIZE_HOOK_OUTPUT ) + PROTOCOLLIST_BUFFER_MAXSIZE )

	// --> IOCTL_VPCKNT_SUBMIT_OALIST <--

	typedef struct _OALIST_ITEM
	{
		DWORD		dwOpenAdapterOrdinal;

	} OALIST_ITEM, *POALIST_ITEM;

	// --> IOCTL_VPCKNT_GET_NEXT_PACKET <--

	typedef struct _HOOK_STATS
	{
		// === STATISTICS ===

		DWORD			dwLostPackets;
		
		BYTE			bArePacketsRemaining;

		//=================================================================================
		// If sending a IOCTL_VPCKNT_GET_NEXT_PACKET then a NEXT_PACKET structure follows.
		//=================================================================================

	} HOOK_STATS, *PHOOK_STATS;

	typedef struct _NEXT_PACKET
	{
		// === IDENTIFICATION ===

		DWORD			dwOpenAdapterOrdinal;

		// === ORDERING ===

		DWORD			dwPacketSerial;

		// === PACKET DATA ===

		BYTE*			pbData;
		DWORD			dwDataLength;

		BYTE			bDirection;

		//===============================================================================
		// if pbData == NULL then the packet data is placed at the end of the structure.
		//===============================================================================

	} NEXT_PACKET, *PNEXT_PACKET;

#pragma pack(pop)

//======================
// General Definitions.
//======================

#define FILE_DEVICE_UNKNOWN             0x00000022
#define IOCTL_UNKNOWN_BASE              FILE_DEVICE_UNKNOWN

//================================================
// Definitions about the Driver Version and Name.
//================================================

#define SYSDRIVER_NAME_ANSI			"VPCKnt"	// Name of the Driver and Device.
#define SYSDRIVER_NAME_WIDE			L"VPCKnt"	//   "      "      "      "

#define _VPCKnt_DRIVER_VERSION_		( 1 )		// Driver Version actually in use.
